<?php

/*
  affiliate.php
  Required for the ISC Affiliate Module by C. NEDELCU
  http://cnedelcu.net


  Modified by Martin ( http://www.snailsolutions.co.uk )
  - Use ISC 6.1.x order variables
  - Commission details now record actual commission rate instead of hard coded 5%
  - Set AFFILIATE_REASON to AFFILIATE-CREDIT
  - Only allow members of affiliate group to get credit or access affiliate menu
  - Variable commission rate dependant on the order value


  Set variables below to configure the module
 */

define("AFFILIATE_ENABLED", true);  // enable or disable the affiliate module
define("AFFILIATE_COOKIE_DURATION", 14);   // how many days should the affiliate cookie remain active?

define("AFFILIATE_FIXED_AMOUNT", 0);  // if you want to use a fixed amount IN ADDITION TO THE COMMISSION, define the amount here.
define("AFFILIATE_REASON", "AFFILIATE-CREDIT");  // When an order is made, an entry in the "customer_credit" table is added. This is the credit "reason". If you change this after some sales were made, the sales won't be listed (see DisplayAffiliateAccountPage function below)
define("AFFILIATE_SELF_AFFILIATE", false);  // If the affiliate account is the same as the customer account (the customer is the affiliate) do we credit the affiliate account?
define("AFFILIATE_INCLUDE_SHIPPING", false);  // Include shipping costs in the commission calculation?

define("AFFILIATE_CUSTOMER_GROUP_NOVAT", 14);  // Customer group of affiliates (not VAT registered)
define("AFFILIATE_CUSTOMER_GROUP_VAT", 12);   // Customer group of affiliates (VAT registered)

/*
 * define( "AFFILIATE_COMMISSION", 		0); 	// commission that affiliates make when a sale is made. Here it's 5%
 * 
 * This has been deprecated and replaced with the commission_rates array to allow a graded commission
 * based on the order value.
 */


/*
 * Set Commission rates by order value
 * array( value (or lower) => commission_rate )
 */
function set_commission_rates() {
	return array(
		100 => 6,
		40 => 5,
		20 => 4,
		10 => 3,
		5 => 2
	);
}


/*
 * Determine commission rate being given based on value of order
 */

function getCommissionRate($value) {
	$commission_rates = set_commission_rates();

	//$commission_rates = ksort($commission_rates);
	
	if(!is_array($commission_rates)) {
		$GLOBALS['ISC_CLASS_LOG']->LogSystemDebug("general", "commission_rates array not set");
		return 0;
	}

	foreach ($commission_rates AS $limit => $rate) {
		if ($limit <= $value) {
			$GLOBALS['ISC_CLASS_LOG']->LogSystemDebug("general", "commission_rates set: {$rate}");
			return $rate;
		}
	}
	$GLOBALS['ISC_CLASS_LOG']->LogSystemDebug("general", "commission_rates - rate not set");
	return 0;
}

/*
 * Check if the customer is in one of our affiliate customer groups
 * Stops non-affiliate customers from accessing and gaining from affiliate discounts
 */

function checkAffiliate($customerid) {

	$customerid = intval($customerid);
	if (empty($customerid)) {
		return false;
	}

	$GLOBALS['ISC_CLASS_CUSTOMER'] = GetClass('ISC_CUSTOMER');
	$customer = $GLOBALS['ISC_CLASS_CUSTOMER']->GetCustomerInfo($customerid);

	if ($customer['custgroupid'] == AFFILIATE_CUSTOMER_GROUP_NOVAT) {
		return true;
	} elseif ($customer['custgroupid'] == AFFILIATE_CUSTOMER_GROUP_VAT) {
		return true;
	}
	return false;
}

// This function will be called when the affiliate must be given the credit
function GiveAffiliateCredit($order, $cancelled=false) {

	if (!AFFILIATE_ENABLED) {
		return false;
	}

	///////////// SET VARIABLES /////////////
	$REASON = AFFILIATE_REASON;		// 
	$AFFILIATE_CAN_BE_BUYER = AFFILIATE_SELF_AFFILIATE; // 
	/////////////////////////////////////////
	// Get some data from request
	$oid = intval($order['orderid']);

	// What is the total? does it include shipping costs?
	$amount = AFFILIATE_INCLUDE_SHIPPING ? $order['total_ex_tax'] : $order['subtotal_ex_tax'];
	$date = time();
	$aid = intval($order['ordaffiliate']);
	$cid = intval($order['ordcustid']);

	// AID not set ? (affiliate id)
	if (!$aid) {
		// if this happens, it's likely that you didnt change the SQL table properly
		return false;
	}

	// the affiliate is the customer itself?
	if ($aid == $cid && !$AFFILIATE_CAN_BE_BUYER) {
		return false;
	}

	// Is the affiliate_id that an affiliate group member?
	if (!checkAffiliate($aid)) {
		return false;
	}

	$RATE = getCommissionRate($amount) / 100;

	// Vars for cancellation full or partial
	$credited = $diff = 0;
	
	// Has there already been a credit for this order?
	$query = "SELECT creditamount FROM isc_customer_credits WHERE creditrefid=$oid";
	$result = $GLOBALS['ISC_CLASS_DB']->Query($query);
	if ($GLOBALS['ISC_CLASS_DB']->CountResult($result) > 0) {
		
		if($cancelled == false) {
			// Not cancelling so we don't want to credit again
			return false;
		}
		else {
			$row = '';
			$credited = round($GLOBALS['ISC_CLASS_DB']->FetchOne($result), 2);
		}
	}
	elseif($cancelled === true) {
		// Cancelling so if there's no credit, we're already done
		return false;
	}
	
	// Define the amount: only $RATE% of request
	$credit_amount = round($amount * $RATE, 2);
	$credit_amount += AFFILIATE_FIXED_AMOUNT; // add the fixed amount, 0 by default
	
	/*
	 * If we're cancelling check the amount remaining is the same as the amount
	 * on the order
	 * 
	 * TODO figure out what happens with a partial refund (manual adjustment for now!)
	 */
	if($credited > 0) {

		$query = "DELETE FROM isc_customer_credits WHERE creditrefid=$oid";
		$diff = $credited;
	}
	else {
	
	// Execute query
		$query = "
			INSERT INTO isc_customer_credits
				(customerid, creditamount, credittype, creditdate, creditrefid, credituserid, creditreason)
			VALUES (
				'$cid',
				'$credit_amount',
				'gift',
				'$date',
				'$oid',
				'$aid',
				'" . $GLOBALS['ISC_CLASS_DB']->Quote($REASON) . "'
			)";
	}
	
	if($cancelled) {
		$GLOBALS['ISC_CLASS_LOG']->LogSystemDebug("general", "Affiliate credit changes", 
			"SQL:  $query <br> ORDER_CREDIT: $credit_amount, CREDITED: $credited, DEDUCT: <strong>$diff</strong>");
	}
	
	$GLOBALS['ISC_CLASS_DB']->Query($query);

	// Did an error occur?
	
	$error = $GLOBALS['ISC_CLASS_DB']->Error();
	if ($error) {
		return false;
	}

	// Update store credit for the customer
	$query = "UPDATE isc_customers SET custstorecredit=custstorecredit";
	$query .= ($diff > 0) ? "-{$diff}" : "+{$credit_amount}";
	$query .= " WHERE customerid=$aid";
	
	$GLOBALS['ISC_CLASS_LOG']->LogSystemDebug("general", "Affiliate credit changed", "SQL: $query");
	$GLOBALS['ISC_CLASS_DB']->Query($query);

	return true;
}



// Display commission details
function GetCommissionDetails() {
	$commission_rates = set_commission_rates();

	$out = "";
	$curr = GetDefaultCurrency();

	if (is_array($commission_rates) & count($commission_rates) >= 1) {
		$out = '';
		foreach ($commission_rates AS $value => $rate) {
			$out .= $rate . "% on order values of ".FormatPriceInCurrency($value, $curr)." (ex tax) or more<br/>";
		}
		$out .= AFFILIATE_INCLUDE_SHIPPING ? "" : "(Excludes shipping/handling costs)<br/>";

		if (AFFILIATE_FIXED_AMOUNT) {
			$out .= " and a fixed " . FormatPriceInCurrency(AFFILIATE_FIXED_AMOUNT, $curr) . " bonus";
		}
	} elseif (AFFILIATE_FIXED_AMOUNT) {
		$out = "a fixed " . FormatPriceInCurrency(AFFILIATE_FIXED_AMOUNT, $curr) . " bonus";
	} else {
		die("The affiliate module is misconfigured (0% commission, 0 bonus)");
	}
	return $out;
}

// Display the affiliate account page
function DisplayAffiliateAccountPage() {
	//global $GLOBALS;
	// Customer ID
	$cid = $GLOBALS['ISC_CLASS_CUSTOMER']->GetCustomerId();

	if (!checkAffiliate($cid)) {
		exit;
	}
	
	$cust['custstorecredit'] = empty($cust['custstorecredit']) ? 0 : $cust['custstorecredit'] * 1;

	// Get current store credit
	$cust = $GLOBALS['ISC_CLASS_CUSTOMER']->GetCustomerDataByToken();
	$credit = FormatPriceInCurrency($cust['custstorecredit'], GetDefaultCurrency());

	// List sales
	$query = "
		SELECT * 
		FROM isc_customer_credits c
		WHERE credituserid='$cid'
		AND creditreason='" . mysql_real_escape_string(AFFILIATE_REASON) . "'
		ORDER BY creditdate DESC
	";
	$result = $GLOBALS['ISC_CLASS_DB']->Query($query);
	$totalcredit = 0;
	$sales = "";
	while ($row = $GLOBALS['ISC_CLASS_DB']->Fetch($result)) {
		$date = date("Y-m-d", $row['creditdate']);
		$ocredit = round($row['creditamount'], 2);
		$fcredit = FormatPriceInCurrency($ocredit, GetDefaultCurrency());
		$oid = $row['creditrefid'];
		$sales .= "
			<li>[{$row['custcreditid']}] On $date, you received <b>$fcredit</b>. Order ID: $oid.</li>
		";
		$totalcredit += $ocredit;
	}
	$totalcredit = FormatPriceInCurrency($totalcredit, GetDefaultCurrency());
	//$credit = FormatPriceInCurrency($credit, GetDefaultCurrency());
	$example = $GLOBALS['ShopPath'] . "/?ref=$cid";

	// Set up affiliate account page
	$GLOBALS['AffiliateAccount'] = "
	<hr />
	<div style='text-align: justify;'>
	<h2>My Affiliate Account</h2>
	
	<p>As an affiliate you will attract the following credit/rewards to your own customer account as follows<br>
	" . GetCommissionDetails() . ".</p>
	
	<h3>Technical Info'</h3>
	<p>To make use of your affiliate account you simply share appropriate links to our website.</p>

	<p>Affiliate links can be created by appending <u>?ref=$cid</u> to any link to our website, products or categories.
	<br>Example: <u>$example</u></p>

	<p>If a customer clicks on one of your affiliate enabled links <em>(see above)</em> then places an order within the next " . AFFILIATE_COOKIE_DURATION . " days, 
	you will receive the corresponding store credit once the order has been completed.
	<br><em>Note: Your customers browser needs to accept cookies for the store when using the affiliate link for it to work</em></p>
	
	<h3>Payment/Store Credit Use</h3>
	<p>You can then either use your store credit to make purchases on our website, or contact us
	to arrange payment.</p>
	
	<h3>Terms and Conditions</h3>
	<ul>
	<li>Payment of any credit requires a valid invoice from you for the amount you wish to receive</li>
	<li>Payment of any credit may also require a waiting period of up to 30 days to allow for any refunds, cancellations, etc..</li>
	<li>Orders that resulted from a referral that are subsequently cancelled, refunded, etc.. will result in
	any related credit being deducted from your balance</li>
	<li>We reserve the right to suspend any credit accrued if the use of the referral account does not meet with 
	our companies customer centric policies or lack basic common sense (eg: misselling, misrepresentation, spam linking, etc...)</li>
	<li>We reserve the right to amend these terms and conditions without notice</li>
	<li><strong>Your inclusion on this referral program is primarily aimed at enabling some small reward or thank-you for
	your willingness to send customers our way because you <u>already</u> believe in our product(s) and service(s).</strong><br><br>
	Please do not recommend services or products if you do not honestly believe they represent the best solution
	to your recipient.</li>
	</ul>
	
	<h3>Your Affiliate Sales</h3>
	<p>Your current store credit balance is: <b>$credit</b></p>
	<p>To date, you have accumulated <b>$totalcredit</b> in store credit thanks to your affiliate sales.</p>
	<ul>$sales</ul>
	</div>
	";
}

/* INITIALIZE COOKIE */
if (isset($_GET["ref"]) && AFFILIATE_ENABLED) {
	$ref = intval($_GET["ref"]);
	ISC_SetCookie("ref", $ref, time() + 3600 * 24 * AFFILIATE_COOKIE_DURATION);
}
